#ifndef _FLOW_CACHE_H
#define _FLOW_CACHE_H

#include "flow.h"
#include "sfxhash.h"

typedef struct _FCSTAT
{
    uint64_t find_ops;
    uint64_t reversed_ops;
    uint64_t find_success;
    uint64_t find_fail;
    uint64_t new_flows;
    uint64_t released_flows;
} FCSTAT;

typedef struct _FLOWCACHE
{
    SFXHASH *ipv4_table;
    /* statistics */
    FCSTAT total;            /* statistics for everything */
    FCSTAT per_proto[256];   /* statistics kept per protocol */
} FLOWCACHE;


int flowcache_init(FLOWCACHE *flowcachep, unsigned int rows, int memcap,
                   FLOWHASHID hashid);
int flowcache_destroy(FLOWCACHE *flowcachep);
int flowcache_releaseflow(FLOWCACHE *flowcachep, FLOW **flowpp);
int flowcache_newflow(FLOWCACHE *flowcachep, FLOWKEY *keyp, FLOW **flowpp);
int flowcache_find(FLOWCACHE *flowcachep, FLOWKEY *keyp,
                   FLOW **flowpp, int *direction);

void flowcache_stats(FILE *stream, FLOWCACHE *flowcachep);

int flowcache_overhead_bytes(FLOWCACHE *fcp);

int flowcache_memcap(FLOWCACHE *fcp);
int flowcache_row_count(FLOWCACHE *fcp);

/* utilty functions */
const char *flowcache_pname(FLOW_POSITION position);

#endif /* _FLOW_CACHE_H */
